function  [stts]= check_if_point_belongs_to_object( varargin )
% chech if given point (x,y,z) is a part of object. Margin is
% for computation accuracy (rounding object to mesh) usualy half of grid step in each coordinate.
% varargin{1} - object
% varargin{2} - vector with x y z coordinates
% varargin{3} - vector with z r alfa margins 
% output_args = 1 (true) if point belongs to object otherwise 0 (false)

   %             geometry.rmin= rmin_mm [milimeters] radius-inner part of element
   %             geometry.rmax= rmax_mm [milimeters] radius-outer part of element
   % 
   %             geometry.start_angle_deg= start_angle [degree] where object begins
   %             geometry.stop_angle_deg= stop_angle [degree] where object ends
   % 
   %             geometry.h_high= upper height of the element (z-axis)
   %             geometry.h_low

this=varargin{1};
vector_xyz=varargin{2};
margin_zra=varargin{3};

x=vector_xyz(1);
y=vector_xyz(2);
z=vector_xyz(3);

zm=margin_zra(1);
rm=margin_zra(2);
am=margin_zra(3);

obj_z_high = this.geometry.h_high;
obj_z_low  = this.geometry.h_low;

obj_z_highm=obj_z_high+zm;
obj_z_lowm=obj_z_low-zm;

if obj_z_lowm<=z && z<=obj_z_highm
    
    obj_r_high = this.geometry.rmax;
    obj_r_low  = this.geometry.rmin;
    
    obj_r_highm = obj_r_high + rm;
    obj_r_lowm  = obj_r_low - rm;
        
    xc=this.geometry.x_centre;
    yc=this.geometry.y_centre;
    
    % outside side
    if ((x-xc)^2 + (y-yc)^2<= obj_r_highm^2)
       
        if ((x-xc)^2 + (y-yc)^2 >= obj_r_lowm^2)
            
            x_old=x;
            y_old=y;
            
            if x>=0 || y>=0
                x=x-xc;
                y=y-yc;
                        
            elseif x<=0 || y>=0
                x=x+xc;
                y=y-yc;            
                
            elseif x<=0 || y<=0
                x=x+xc;
                y=y+yc;            
            
            elseif x>=0 || y<=0
                x=x-xc;
                y=y+yc;
            end
            
            [TH,R]=cart2pol(x,y);
            % cart2pol returns:
            % THETA is a counterclockwise angular displacement in radians from the positive x-axis
            % this is not exactly what we want    TH <-pi;pi>

            TH = TH * (180/pi); % transform to degrees

            TH = mod(TH,360); % keep it 0<=TH<360

            if TH<0
                TH=TH+360;                
            end
            
            obj_a_start = this.geometry.start_angle_deg;
            obj_a_stop  = this.geometry.stop_angle_deg;
        
            obj_a_startm = obj_a_start -am;
            obj_a_stopm  = obj_a_stop +am;
         
            if obj_a_startm<0
                if TH>=360+obj_a_startm                    
                    TH=TH-360;                    
                end
            end
            
        if TH >= obj_a_startm && TH <=obj_a_stopm;

           stts=1;
%            hold on;
%            plot3(x_old,y_old,z,'ro');
           
        else 
            stts=0;
        end
        
        else
            stts=0;
        end              
       
    else 
        stts=0;
    end

    
else 
    stts=0;
end

end

