classdef eFwd_Problem < handle
    %EFWD_PROBLEM class describes forward problem. Usue to compute
    % sensitivity maps (S) and capacitance (C)
    % C=S*e where e=permittivity map
    
    properties
        potential_maps_3d_matrix   % potential maps for all electrodes. 
                                   % num_of_electrodes x potential map resize to 3d matrix 
        S_map            % sensititvity map computed using potential_maps. Ther are values
                         % for each electrode pair. 
                         % 4dim_matrix : pair_num x sens_map resize to 3d matrix 
        S_matrix         % sensitivity maps reshape from 3d matrix to single vectors-> 
                         % matrix cols=num_of_sens_maps, in each col
                         % sensitivity map for selected points (usualy points where potential was computed)
        S_map_resize     % resized (down scale) sensitivity maps
        permittivity_3d_matrix     % vector of permittivity of reconstructed object
        C                % capacitance of each electrode C=S*e 
        mesh             % size (x y z) of mesh in voxels 
        point_list       % linear index for those points where potential computing have been computed
        index_which_sensitivity_maps_to_correct % this cell array contains in each column index for 2d sensitivity maps for constant space
                                                % [electrode_al elestrode_r sensitivity_map_index]
        vacuum_permittivity=(1/(36*pi))*1e-9;   % [F/m]   
        corr='none'      % geometrical correction value for sensitivity matrix
        measurements_all=0 % specified if sensitivity computation will be done with repeats or without
        num_of_electrodes % number of electrodes
        el_a_vect         % aplication electrodes number
        el_r_vect         % receiving electrodes number
    end
    
    methods (Access='public')
        function this = eFwd_Problem(varargin)
          if nargin > 0 ; % no input arguments
          if isa(varargin{1},'eFwd_Problem')
            this = varargin{1}; % copy constructor
            display 'Copy constructor eFwd_Problem'
            return;                     
          else
             switch(nargin)                                  
              
                case 4,                                
                this.mesh=varargin{1};                
                this.potential_maps_3d_matrix=varargin{2}; 
                this.permittivity_3d_matrix=varargin{3};
                this.point_list=varargin{4};
                
%                 matrix_size=size(this.potential_maps_3d_matrix);
%                 
%                 num_of_electrodes=matrix_size(1);
%                 
%                 single_sensitivity_map_size=matrix_size(2:end);
%                 
%                 num_of_sens_map=(num_of_electrodes*(num_of_electrodes-1))/2;
%                 
%                 this.S_map=zeros([num_of_sens_map single_sensitivity_map_size]);
                
                 otherwise,
                 error('Too less or too much input elements');
             end % switch
          end % if
          end % if
          end % constructor
          
          % compute sensitivity maps
          % output sensitivity maps
          S_map=compute_S_map(varargin)
          
          % compute capacitance for each electrode
          % output capacitance
          C=compute_C(varargin)
          
          % resize sensitivity maps
          % output sensitivity maps
          S_map_resize=resize_S_map(varargin)
          
          % plot sensitivity maps
          plot_S_map(varargin)
         
          [ S_corrected ] = geometric_correction(varargin)
          % Geometrical correction of sensitivity maps. Electrodes with the same symetric length
          % should have the same values 
          % input: matrix with electrodes pair and sensitivity maps (constant electrode length in each column)
          %      : sensitivity map
          % output: corrected sensitivity map
    
          plot_S_map_slice( varargin )
         % PLOT_S_MAP_SLICE plot specific slice in specific axes
         %   input: el_r
         %          el_a
         %          axis in 'r' 'c' 'p'
         %          point in space (matrix indicies)  [12 10 15]
         %          Fp.plot_S_map_slice(1,16,'p',12);
         
         [el_a el_r] = electrodes_pairs( varargin )
        %creating of electrode application and receiving pairs
        % input num of electrodes
        % output electrodes outputs aplication 
        %                   and receiving

    end % methods public
    
    methods (Access='private')

            %input: potential map first electrode
            %       potential map second electrode
            %       potential difference of electrodes
            % return S (Sensotivity map) for set electrodes potential maps
            S_map_of_pair=compute_S_electrode_pair(varargin);
        
    end % private methods
end

