classdef eInv_Problem < handle
    %EINV_PROBLEM describes inverse problem in Electrical Capacitance
    %Tomography. Using capacitance (C) we compute permittivty (e)
    
    properties (SetAccess='public')
        C_pha % capacitance measure for phantom
        C_min % capacitance with minimal values in FOV
        C_avg % capacitance with average values in FOV
        C_max % capacitance with maximal values in FOV
        
        FP_min % forward problem solve for min value in FOV
        FP_max % forward problem solve for max value in FOV
        %FP_avg % forward problem solve for average value in FOV
        FP_phantom % forward problem solve for phantom 
        mesh_invp='none'; % mesh object with size to reconstruct
        
        permitivity_phantom % permittivity of phantom
        permittivity_reconstructed_in_each_iteration % permittivity reconstructed in each iteration
        S_use_to_reconstruction % sensitivity use to reconstruction
        S_phantom % model sensitivity 
        mesh_voxel_size_upscale % size of upscale model
        mesh_voxel_size_downscale % size of downscale model
        number_of_iteration % number of iteration which have been done
        reconstruction_parameters % parameters of reconstruction
        
        sensitivity_sum_index % vector contains point_list where sensitivity 
                              % was sum from 3d to 2d
        reshape_S_map_2d      % sum of sensitivity maps from 3d to 2 maps. 
                              % matrix= electrode combination x vector
                              % sensitivity
                              % reshape_S_map_2d(:,k) where k is electrode
                              % combination
                              
        S_matrix_2d           % vector contains 2d matrix for each electrode pair 
                              % S_matrix_2d(k,:,:) where k is electrode
                              % combination
    end
    
 methods (Access='public')                
    
    permittivity=upscale_eps(varargin)
    % upscale permittivity to bigger model (forward problem)
    
    S_downscale=downscale_S(varargin)    
    % resize sensitivity to reconstruction size
    % input S_3d matrix to rescale
    
    S=upscale_S(varargin)
    % resize sensitivity to forwad problem size
    
    compare_permittivity(varargin)
    % compare two permittivity vectors
     
    compare_S(varargin)
    % compare two sensitivity vectors
    
    C=get_C_from_file(varargin)
    % input file name which containc capacitance (from ECT3 tomography)
    
    [reshape_S_map_2d indicies_2d S_matrix_2d]=sum_sensitivity_maps_to_2D(varargin)
    % sum sesnitivity maps fro 3d to 2d in specific volume
    % input: point list of volume where maps will be sum
    
    plot_2d(varargin)
    % plot 2d plots only 2d images (it can be use to plot sensitivity o permittivity maps or matrix).
    % Input: S map or matrix to plot 
    %        optional resize vector
    %        plot_mode ('mesh' or 'imagesc')
    % e.g.   this.plot_2d(S_sum,[30,30],'mesh')

    sum_S_map=sum_s_map_to_one_map(varargin)
    % sum s_maps to one map
    % input matrix 
    % S_sum=this.sum_s_map_to_one_map(reshape_S_map_2d);
    % this.plot_2d(S_sum,[reshape size],'mesh');
    
    mtrx_2d=reshape_to_2d(varargin)
    % reshape 4 or 3 dim arrays (electrode combination + 3 or 2 dim array) to 2d array (electrode combination + vector)
    % input 4 or 3 dim matrix
    
    plot_min_max_measure_capacitance(varargin)
    % plot minimal maximal and mesured capacitance values
    
    S_map_ret=get_from_maps_specific_points(varargin)
    % get specific points from input 4d maps
    % return the same 4d maps but only with values from point list
    
    matrix_2d=convert_map_to_mtrx(varargin)
    %convert map to matrix (2 dimension), each map in each row
    % input: map
    %        point_list_which_select
    % output: mtrx
    
    [ linear_in_2d ] = linearind_3d_to_2d( varargin)
    % convert linear ind of 3d matrix to linear indexs of 2d matrix
    % input
    
    [ map_ret ] = get_points_and_reshape( varargin )
    % GET_FROM_MAPS_SPECIFIC_POINTS get from input maps only this points which
    % are in input of function. Return maps with the 2d matrix size
    % input 4d matrix first number is map number
    %       point_list
    % output matrix 2d
    
 end % methods public
    
 methods (Abstract=true, Access='public')    
        permittivity=solve(varargin)
        % Use to solve inverse problem. Usualy should contain algorithm
        % implementation.
 end
    
end

