classdef eMaterial< handle
    % Class describes material of structural element use to build
    % numerical object which represents real objects
    % more(folder which contains ECT package must be on path): 
    %     doc eMaterial   
    % example use:
    % clear all;
    % close all;
    % import ECT.* % parent folder of ECT package must be on Path
    % name='dielectric';
    % name_p = {'perm' 'cond'};
    % val=[1,2];
    % dielectric=eMaterial(name,name_p,val);
    % 
    % material_parameter_name={'perm','temp'};
    % value=[2,3];
    % dielectric.set_material_parameter(material_parameter_name, value);
    % 
    % dielectric.remove_material_parameter('cond');
    % 
    % dielectric=eMaterial('wizard');    
    
    properties (SetAccess='public')
        name                            % material name e.g. Dielectric
        material_parameter_name_cell    % vector which describes name of material parameters e.g (permitivity)
        material_parameter_value_vector % vector which describes values of material parameters
        unit_vector                     % unit
    end
    
    methods (Access='public')
        
        function M_obj=eMaterial(varargin)            
        % constructor: param_name_vector, value_name_vector
        % example:
        % name='dielectric'
        % param_name = {'perm' 'cond'}; % cell
        % val=[1,2]; % vector
        % dielectric=eMaterial(name,param_name,val);
        % or use wizard:
        % dielectric=eMaterial('wizard');
        
            if nargin > 0 ; % number input arguments
              if isa(varargin{1},'eMaterial')
                M_obj = varargin{1}; % copy constructor
                display 'Copy constructor eMaterial'
                return;                     
              else                                           
                   switch(nargin)
                       case 1,
                           if(strcmp(varargin{1},'wizard'))                               
                               name_val=input('Material name:', 's');
                                                                                                                              
                               % values shouldn't be empty 
                               name_tmp = {'none'};
                               val_tmp=0;

                               M_obj.material_parameter_name_cell=name_tmp;
                               M_obj.material_parameter_value_vector=val_tmp;
                               
                               reply = 'Y';
                               while strcmp(reply,'Y')
                                                                   
                                param_name=input('Material parameter name e.g. permittivity: ', 's');
                                
                                if isempty(param_name)
                                    error('empty value')
                                end
                                
                                value=input('Material parameter value e.g.100: ');
                                
                                if isempty(value)
                                    error('empty value')
                                end
                                
                                C_param_name = cellstr(param_name);

                                
                                M_obj.set_material_parameter(C_param_name, value);
                                
                                reply = input('Do you want more? Y/N [Y]: ', 's');
                                if isempty(reply)
                                    reply = 'Y';
                                end
                                
                               end                               
                              
                              M_obj.name=name_val;
                              M_obj.remove_material_parameter('none');

                           end
                       case 3,  
                        
                          name_val=varargin{1};
                          param_name=varargin{2};
                          value=varargin{3};

                          pns=size(param_name);
                          vns=size(value);
                          
                          if (pns<0) OR (vns<0)
                              error('input is empty')
                          else
                        
                             if pns==vns
                              
                              M_obj.name=name_val;
                                 
                              M_obj.material_parameter_name_cell = cell(pns);
                              M_obj.material_parameter_value_vector=zeros(pns); 

                              M_obj.material_parameter_name_cell(:)=param_name(:);
                              M_obj.material_parameter_value_vector(:)=value(:);

                             else
                              error('Param name vector and value size vector size must be the same');
                             end
                          end
                          
                    otherwise,
                             error('Too less or too much input elements');
                   end % switch                                                                              
                  end
              end
            end%constructor        
    end %method
    
    
    methods (Access='public')
        
        
        function [sucess]=set_material_parameter(M_obj,material_parameter_name, value)   
        % Use to set or update value for material prametrs e.g. set_material_parameter('permittivity', 100)
        % example
        % material_parameter_name={'perm','temp'};  % cell
        % value=[2,3]; % vector
        % dielectric.set_material_parameter(material_parameter_name, value);
            
            smpn=size(material_parameter_name);
            sv=size(value);                                                
            
            if smpn==sv
            
            % map to add
            mapObjtoAdd = containers.Map(material_parameter_name,value);               
                
            % old map          
            mapObj = containers.Map(M_obj.material_parameter_name_cell,M_obj.material_parameter_value_vector);
              
            warning off
            % remove the same elements from old map which are in new map
            remove(mapObj, material_parameter_name);      
            warning on
            
            % sum of maps old + new one
            sumMap = [mapObj; mapObjtoAdd];
                                    
            M_obj.material_parameter_name_cell=sumMap.keys;
            M_obj.material_parameter_value_vector=sumMap.values;
            
            sucess=1;
            
            else
                sucess=0;
                error('parameter name and value must have the same size')
            end
            
        end
               
       
        function [sucess]=remove_material_parameter(M_obj,material_parameter_name)   
        % Use to remove material prameters e.g. remove_material_parameter('permittivity')
        % this function remove parameter and value    
        
            % map          
            mapObj = containers.Map(M_obj.material_parameter_name_cell,M_obj.material_parameter_value_vector);
                 
            % remove elements from map
            remove(mapObj, material_parameter_name);                                       
                                    
            M_obj.material_parameter_name_cell=mapObj.keys;
            M_obj.material_parameter_value_vector=mapObj.values;
            
            sucess=1;                      
            
        end
        
    end %methods
        
end