function  [return_vals]=calculate_model(varargin)
% calculates boundary and permittivity maps
%
% object variable boundary_permittivity_neighbours is set
% boundary_permittivity_neighbours contains: 
% pointlist 
% permittivity 
% start potential of each discrete point 
% neighbours pidx 
% neighbours iidx
% potential boundaries
% 
% input:
% change_value_object_name   name of the object that will be changing
% values e.g. 'electrode'
% potential_on   values of the object e.g. electrode when potential is on
% potential_off  values of the object e.g. electrode when potential is off
% potential__others_start_vals   start potential values of no boundary objects
% perm_min_in_FOV use to create min permittivity map
% perm_max_in_FOV use to create max permittivity map
%
% output:
% return_vals=[matrix_mesh_size boundary_permittivity_neighbours all_boundary_change_val_object_iidx_and_pidx separate_change_val_object_iidx_pidx];
%

this=varargin{1};
change_value_object_name=varargin{2};
potential_on=varargin{3};
potential_off=varargin{4};
potential_others_start_vals=varargin{5};
perm_min_in_FOV=varargin{6};
perm_max_in_FOV=varargin{7};


mesh=this.mesh;
% e.g.: electrodes changing his potential value 

linear_indicies=this.all_elements.get_linear_indicies();

%alloc
this.boundary_permittivity_neighbours=zeros([length(linear_indicies) 3]);

%fill with '-1'
this.boundary_permittivity_neighbours(:,:)=-1;

this.boundary_permittivity_neighbours(:,1)=linear_indicies(:);

%alloc
%permittivity_tmp=ones(length(linear_indicies),1).*(-1);

tmp=mesh.get_neighbours(1);
this.neighbours_pidx=ones(length(linear_indicies),length(tmp)).*(-1);
this.neighbours_iidx=ones(length(linear_indicies),length(tmp)).*(-1);
%point_list_model=cell2mat(this.boundary_permittivity_neighbours(:,1));
point_list_model=linear_indicies;

model_elements=this.all_elements.get_all();

len=length(model_elements);

% Set with object which boundry values changing
set_bundary_val_change=eSet('boundary_val_change');

p=1;
display 'calculate model'
set_boundary=eSet('boundary_points');
set_non_boundary=eSet('boundary_points');


for i=1:len
    boundary_set=model_elements(i).is_boundary;
    
    if boundary_set==1
     set_boundary.add_element(model_elements(i));
    else
     set_non_boundary.add_element(model_elements(i));
    end
end

% remove point from non boundary objects
boundary_ind=set_boundary.get_linear_indicies();
non_boundary_ind=set_non_boundary.get_linear_indicies();
iidx_share_points= ismember(boundary_ind,non_boundary_ind);
share_points_values=boundary_ind(iidx_share_points);


%update set senor with object with computed point lists 
 non_boundary_keys=set_non_boundary.element_map.keys;
% get from all meshed objects only sensor objects 
 non_boundary_keys=cell2mat(non_boundary_keys);

for i=1:numel(non_boundary_keys)
    non_B_element(i)=this.all_elements.element_map(non_boundary_keys(i));
    this.all_elements.remove_element(non_boundary_keys(i),'id');
    
    pl_non_b=non_B_element(i).point_list;
    
    not_same_ind=~ismember(pl_non_b,share_points_values);
    
    pl_non_b=pl_non_b(not_same_ind);
    
    non_B_element(i).point_list=pl_non_b;
    
    this.all_elements.add_element(non_B_element(i));
end

% !!!! przepisa do sensora i phantomu all elemnts


display 'set boundary and permittivity'
for i=1:len
    i
    point_list=model_elements(i).point_list;
    potential=model_elements(i).potential;
    permittivity=model_elements(i).material.material_parameter_value_vector(1);
    boundary_set=model_elements(i).is_boundary;
    object_name=model_elements(i).name;
    %id=model_elements(i).element_id;
    
    % add to Set with object which boundry values changing
    if strcmp(object_name,change_value_object_name)
        set_bundary_val_change.add_element(model_elements(i));
        pidx_of_object=model_elements(i).point_list;
        
        iidx_of_object=zeros([length(pidx_of_object) 1]);        
        for k=1:length(pidx_of_object)
            iidx_of_object(k)=find(point_list_model==pidx_of_object(k));
        end
        
        tmp=zeros([length(iidx_of_object) 2]);                
        
        iidx_of_object=iidx_of_object';
        tmp(:,1)=iidx_of_object(:);
        
        %pidx_of_object=pidx_of_object';
        tmp(:,2)=pidx_of_object(:);

        separate_change_val_object_iidx_pidx(:,:,p)={tmp};
        % change value object
        p=p+1;
    end
    
    indicies=find(ismember(point_list_model,point_list));
    
    len_pl=length(indicies);
    for k=1:len_pl
%         point_indicies=point_list(k);
%         
%         indicies(k)=find(point_list_model==point_indicies);
        
        %point_list permittivity potential
        model_point_perm=this.boundary_permittivity_neighbours(indicies(k),2);
                
        model_point_potential=this.boundary_permittivity_neighbours(indicies(k),3);        
                
        if model_point_perm==-1
%             display('perm in')
            %this.boundary_permittivity_neighbours(indicies,2)={permittivity};
            %permittivity_tmp(indicies)=permittivity;
            this.boundary_permittivity_neighbours(indicies(k),2)=permittivity;
        else
            
           % old_perm=this.boundary_permittivity_neighbours(indicies,2);
            
%             permittivity=max([old_perm permittivity]);
%             this.boundary_permittivity_neighbours(indicies,2)=permittivity;
            if strcmp(boundary_set,'none')
                   this.boundary_permittivity_neighbours(indicies(k),2)=permittivity;
            else
             res=(model_point_perm+permittivity)/2;
             this.boundary_permittivity_neighbours(indicies(k),2)=res;
            end
        end
        
        if strcmp(boundary_set,'none')
            %do nth
        else
%             display('boundary in')            
            if model_point_potential==-1
                this.boundary_permittivity_neighbours(indicies(k),3)=potential;
            else
                potentials=[model_point_potential potential];
                res=max(potentials);
                this.boundary_permittivity_neighbours(indicies(k),3)=res;            
            end
                         
        end
                  
    end
    
    indicies_alls{i}=indicies;
    indicies=[];
    
end
display 'end set boundary and permittivity'




meshes = repmat(eCube_Mesh,1,len);
meshes(:)=mesh;

display('find neighbours');
parfor i=1:len
        i
        % get neighbours
        %neighbours=[];
        %point_list_all_elements(i,:)=model_elements(i).point_list(:);
        neighbours_all{i}=meshes(i).get_all_neighbours(model_elements(i).point_list);
        %neighbours_all{i}=tmp;
        % if point list doesn't exist value '-1' is return
end
display('end find neighbours');
meshes=[];


models = repmat(eModel,1,len);
models(:)=this;

display('find iidx')
for g=1:len    
    g
    indicies=indicies_alls{g};
    indicies_f{g}=models(g).find_neighbours_iidx(neighbours_all{g},indicies);
    
%    indicies_f{g}=arrayfun(@(nghrs) models(g).find_neighbours_iidx(nghrs),neighbours_all{g});
end
display('end find iidx')

for i=1:len
   point_list=model_elements(i).point_list;
 
    
    indicies=indicies_alls{i};
    neighbours_vect=neighbours_all{i};
        
%     len_pl=length(point_list);
    
%     for k=1:len_pl
%                                
%         neighbours=neighbours_vect(k,:);
%                                                
%         indicies_f(k,:) = arrayfun(@(nghrs) this.find_neighbours_iidx(nghrs),neighbours);                                              
%                 
%     end
    indicies_f_one_object=indicies_f{i};
    
    for r=1:6
        this.neighbours_pidx(indicies(:),r)=neighbours_vect(:,r); 
        this.neighbours_iidx(indicies(:),r)=indicies_f_one_object(:,r);  
    end
    
   % indicies_f=[];
    
end   


% mesh size use to display
matrix_mesh_size=this.mesh.size_discrete_xyz;

%changing values boundary object
boundary_change_val_object_pidx=set_bundary_val_change.get_linear_indicies();
for j=1:length(boundary_change_val_object_pidx)
    boundary_change_val_object_iidx(j)=find(point_list_model==boundary_change_val_object_pidx(j));
end

%alloc memory
all_boundary_change_val_object_iidx_and_pidx=zeros([length(boundary_change_val_object_pidx) 2]);

% first col iidx second pidx
all_boundary_change_val_object_iidx_and_pidx(:,1)=boundary_change_val_object_iidx(:);
all_boundary_change_val_object_iidx_and_pidx(:,2)=boundary_change_val_object_pidx(:);


%boundary_permittivity_neighbours=this.boundary_permittivity_neighbours;

% number of change value object
number_of_change_value_object=p-1;


%alloc
potential_maps=zeros(length(linear_indicies),number_of_change_value_object);

for m=1:number_of_change_value_object

    %alloc
    boundary_start=zeros(length(linear_indicies),1);
    boundary_start(:)=potential_others_start_vals;

    boundary_start(all_boundary_change_val_object_iidx_and_pidx(:,1))=potential_off;

    iidx_pidx_sep=separate_change_val_object_iidx_pidx(m);
    vals=iidx_pidx_sep{1};  
    boundary_start(vals(:,1))=potential_on;
    
    potential_maps(:,m)=boundary_start(:);
end


%alloc

 this.boundary_permittivity_neighbours=[this.boundary_permittivity_neighbours potential_maps];
 boundary_permittivity_neighbours=this.boundary_permittivity_neighbours;
 
 
 % 3d permittivity matrix:

permittivity_matrix_3d=zeros(matrix_mesh_size);
point_list=this.boundary_permittivity_neighbours(:,1);
permittivity=this.boundary_permittivity_neighbours(:,2);


[r c p]=ind2sub(matrix_mesh_size,point_list);

     for i=1:length(point_list)

        permittivity_matrix_3d(r(i),c(i),p(i))=permittivity(i);
        
     end
     

 this.permittivity_matrix_3d=permittivity_matrix_3d;
 this.permittivity=boundary_permittivity_neighbours(:,2);
 
 this.point_list_where_calculate=boundary_permittivity_neighbours(:,1);
 this.boundary=boundary_permittivity_neighbours(:,3);

 
 % get fov linear points
 % put FOV on computed permittivity
fov=this.all_elements.get_elements_by_name({'FOV'});

fov_perm=fov.material.material_parameter_value_vector(1);

values=[fov_perm perm_min_in_FOV perm_max_in_FOV];

% change permittivity get from this.permittivity
[permittivity_out permittivity_3d_matrix_out]=this.change_permittivity_value_in_points(fov.point_list,values);

%addd phantom
%set_fov values for current this.permittivity
this.permittivity_matrix_3d=permittivity_3d_matrix_out{1};
     
this.permittivity=permittivity_out{1};


phantom_elements=this.phantom.get_all();

len=length(phantom_elements);

for p=1:len
    value=phantom_elements(p).material.material_parameter_value_vector(1);
    [permittivity_pha permittivity_pha_3d_matrix_out]=this.change_permittivity_value_in_points(phantom_elements(p).point_list,value);

    this.permittivity_matrix_3d=permittivity_pha_3d_matrix_out{1};
%     
    this.permittivity=permittivity_pha{1};
end

%not necessary
boundary_permittivity_neighbours(:,2)=this.permittivity(:);
this.boundary_permittivity_neighbours=boundary_permittivity_neighbours;


% cell{} with pha, min , max
permittivity_out(1)={this.permittivity};
permittivity_3d_matrix_out(1)={this.permittivity_matrix_3d};

return_vals={boundary_permittivity_neighbours, ...
     this.neighbours_pidx, ... 
     this.neighbours_iidx, ... 
     permittivity_out ,...
     permittivity_3d_matrix_out};
 
 this.all_computed_values=return_vals;


  display 'end calculate model'