clear all;
close all;
 
%  cd mex\
% % -g for debug (delete -g option on production systems)
% %mex('potential_iterative_c.cpp','-g');
% mex('potential_iterative_c.cpp');
% mex -v COPTIMFLAGS="-O2 -DNDEBUG" CFLAGS="-Wall" potential_iterative_c.cpp
% http://msdn.microsoft.com/en-us/library/19z1t1wy.aspx
% cd ..
addpath('mex\');


% %if 0 matlab pool is not running
status = matlabpool('size');
% open as much workers as number of cores
if status==0
    matlabpool('local', feature('numCores'))
end
clear status;

model_params.element_id=0;
figure
model_params=create_phantom_small(model_params);

model_params=create_sensor_small(model_params);

model_params=create_model_small(model_params);

tic
% compute parallel electrical field
parfor i=1:numel(permittivity_3d)
   [EF(i)]=compute_ef(mesh,point_list,permittivity_vector{i},boundary,neighbours_pidx,neighbours_iidx,potential_maps); 
end
toc
ef_pha=EF(1);
ef_pha.show_potential(16);
title 'pha';

ef_min=EF(2);
ef_min.show_potential(1);
title 'min';

ef_max=EF(3);
ef_max.show_potential(1);
title 'max';

status = matlabpool('size');
if status~=0
    matlabpool close
end
% compute parallel forward problem
parfor i=1:numel(permittivity_3d)
   [FP(i)]=compute_fp(EF(i),mesh,point_list,permittivity_3d{i}); 
end

clear FP_phantom
clear FP_min
clear FP_max

FP_phantom = FP(1);
FP_min = FP(2);
FP_max = FP(3);

FP_max.compute_C(model_params.where_compute_capacitance,capacitance_calculation_mode{1});

FP_min.compute_C(model_params.where_compute_capacitance,capacitance_calculation_mode{2});

FP_phantom.compute_C(model_params.where_compute_capacitance,capacitance_calculation_mode{3});

save('G:\model_smaller2');


figure; 
FP_min.plot_S_map_slice(3,11,'p',13);
figure; 
FP_max.plot_S_map_slice(3,11,'p',13);
figure
FP_phantom.plot_S_map_slice(4,11,'p',15);

FP_phantom.plot_S_map(4);

FP_phantom.plot_S_map(14,'single');


%  clear LM;
%  LM=eInv_Problem_LM(FP_phantom,FP_min,FP_max,FP_avg,[20 20 15]);
%  LM.solve('3D',model,model_params.min_perm_value,model_params.max_perm_value,model_params.reconstruct_in_FOV,model_params.reconstruct_in_FOV,100,0.1,'L-curve','modulo',30);

close all
 clear Landweber; 
 Landweber=eInv_Problem_Landweber(FP_phantom,FP_min,FP_max,[30 30]);
 

%  %2d
 invp_fov_size=[32 32 20];
[point_list_invp mesh_invp]=model.getlinear_points_invp('FOV',invp_fov_size,mesh);

  S_map_downscale_min=Landweber.downscale_S(FP_min.S_map,mesh_invp);
 [S_matrix_2d indicies_2d S_map_2d]=Landweber.sum_sensitivity_maps_to_2D(point_list_invp,'p',S_map_downscale_min);
 
 close all
 Landweber.solve('2D','normalize',model_params.min_perm_value,model_params.max_perm_value,100,0.0001,S_matrix_2d, indicies_2d, S_map_2d);

 
clear LBP
close all

LBP=eInv_Problem_LBP(FP_phantom,FP_min,FP_max);

%2D
invp_fov_size=[32 32 20];
[point_list_invp mesh_invp]=model.getlinear_points_invp('FOV',invp_fov_size,mesh);

S_map_downscale_min=LBP.downscale_S(FP_min.S_map,mesh_invp);
[S_matrix_2d indicies_2d S_map_2d]=LBP.sum_sensitivity_maps_to_2D(point_list_invp,'p',S_map_downscale_min);
LBP.solve('2D',model_params.min_perm_value,model_params.max_perm_value,S_matrix_2d, indicies_2d, S_map_2d,point_list_invp,model_params.ins1_point_list, 3);


%3D
close all
invp_fov_size=[32 32 4];
[point_list_invp mesh_invp]=model.getlinear_points_invp('FOV',invp_fov_size,mesh);

S_map_downscale_min=LBP.downscale_S(FP_min.S_map,mesh_invp);
S_matrix_2d = LBP.get_points_and_reshape(S_map_downscale_min, point_list_invp);

indicies_2d=0;
LBP.solve('3D',model_params.min_perm_value,model_params.max_perm_value,S_matrix_2d, indicies_2d, S_map_downscale_min,point_list_invp,model_params.ins1_point_list, 3);

% pomiary
figure('Name','Capacitances: max, min');
v(2:16,1)=FP_max.C(1:15);
v(2:16,2)=FP_min.C(1:15);
% v(2:16,1) = object.max.C(1:15)*5;
% v(2:16,2) = object.min.C(1:15)*5;
bar(v, 'stack');
xlabel('Electrode pair')
ylabel('Capacitance [F]')
grid on
colormap summer


% [r c p]=size(S_map_2d_pha);
% for i=1:r
%     figure(200+i);    
%     map(:,:)=S_map_2d_pha(i,:,:);
%     map=flipud(map);
%     imagesc(map);
%     drawnow    
%     pause(0.5);
% end
% for i=1:r
%     close(200+i);  
% end