classdef Circular_Phantom_Generator < Phantom_Generator
    %CIRCULAR_PHANTOM_GENERATOR generates rods
    %   constructor1: name, 'rods_indx', phantom_construct
    %   phantom_construct(struct):
    %                 phantom_constructor.center_of_FOV_x=0;
    %                 phantom_constructor.center_of_FOV_y=0;
    %                 phantom_constructor.FOV_diameter=30;
    %                 phantom_constructor.d=4;
    %                 phantom_constructor.D=5;
    %                 phantom_constructor.beta=45;
    %                 phantom_constructor.eps_value=30;
    % 
    %                 phantom_constructor.color.r=0; 
    %                 phantom_constructor.color.g=0.5;
    %                 phantom_constructor.color.b=0.6;       
    % 
    %                 phantom_constructor.h_high=40;
    %                 phantom_constructor.h_low=10;
    % 
    %                 phantom_constructor.accuracy=30;
    %                 phantom_constructor.indx=[1 2 3];
    %
    %   constructor2: name, 'wizard'
    
    properties (SetAccess='public')        
        phantom_mode    % in which mode object is creating wizard or not
        construct_obj   % object use to construct CPG in index mode
    end
    
    methods (Access='public')
        function CPG_obj = Circular_Phantom_Generator(varargin)
            
          if nargin > 0 ; % no input arguments
              if isa(varargin{1},'Circular_Phantom_Generator')
                CPG_obj = varargin{1}; % copy constructor
                display 'Copy constructor Circular_Phantom_Generator'
                return;                     
              else
                  
                  switch(nargin)
                  
                  case 2,
                      
                     if(strcmp(varargin{2},'wizard'))
                         CPG_obj.phantom_mode='wizard';
                         CPG_obj.name=varargin{1}; 
                         center_of_FOV_x = input('FOV center x [cartesian coordinates]: ');     
                         center_of_FOV_y = input('FOV center y [cartesian coordinates]: ');     
                         FOV_diameter    = input('FOV diameter [mm]: ');
                         d = input('Circle rod diameter[mm]: ');
                         D = input('D node (circle is in node) [mm]: ');
                         eps_value = input('permittivity value for all rods [mm]: ');
                         beta= input('Rotate phantom [degree]: ');
                         rod_start_high = input('Rods Start High [mm]: ');
                         rod_end_high = input('Rods end High [mm]: ');

                         CPG_obj.vector_phantom_element=repmat(Cylinder_Phantom_Element,1,1);        
                         CPG_obj.number_of_phantom_elements=1;   
                         [X,Y] = CPG_obj.phantom_rod_list(center_of_FOV_x,center_of_FOV_y,FOV_diameter,d,D, beta, eps_value);
                         
                         for i=1:length(X)
                             
                             permittivity=eps_value;  
                             element_name='rode';        
                             element_id=i;
                             color.r=0; 
                             color.g=0.5;
                             color.b=0.6;       
                             
                             geometry.rmin=0;
                             geometry.rmax=d/2; %
                             
                             geometry.start_angle_deg=0;
                             geometry.stop_angle_deg=360;
                             
                             geometry.h_high=rod_end_high;
                             geometry.h_low=rod_start_high;
                             
                             geometry.x_centre=X(i); % center of a cylinder
                             geometry.y_centre=Y(i);
                             
                             geometry.accuracy=30;
                              
                             cpe=Cylinder_Phantom_Element(geometry,permittivity,element_name,element_id,color);
                             
                             CPG_obj.add_element(cpe);
                             
                         end
                         
                     else 
                         error('This is not wizard mode')
                     end
                     
                     case 3,   
                         
                         if(strcmp(varargin{2},'rods_indx'))
                             CPG_obj.name=varargin{1};   

                             CPG_obj.vector_phantom_element=repmat(Cylinder_Phantom_Element,1,1);        
                             CPG_obj.number_of_phantom_elements=1;   
                             CPG_obj.phantom_mode='rods_indx';

                             CPG_obj.construct_obj=varargin{3};                             
                             
                             center_of_FOV_x = CPG_obj.construct_obj.center_of_FOV_x;     
                             center_of_FOV_y = CPG_obj.construct_obj.center_of_FOV_y;     
                             FOV_diameter    = CPG_obj.construct_obj.FOV_diameter;
                             d = CPG_obj.construct_obj.d;
                             D = CPG_obj.construct_obj.D;
                             eps_value = CPG_obj.construct_obj.eps_value;
                             beta= CPG_obj.construct_obj.beta;
                             rod_start_high = CPG_obj.construct_obj.h_low;
                             rod_end_high = CPG_obj.construct_obj.h_high;

                             CPG_obj.vector_phantom_element=repmat(Cylinder_Phantom_Element,1,1);        
                             CPG_obj.number_of_phantom_elements=1;   
                             [X,Y] = CPG_obj.phantom_rod_list(center_of_FOV_x,center_of_FOV_y,FOV_diameter,d,D, beta, eps_value);
                      
                             for i=1:length(X)

                                 permittivity=CPG_obj.construct_obj.eps_value;  
                                 element_name='rode';        
                                 element_id=i;
                                 color.r=CPG_obj.construct_obj.color.r; 
                                 color.g=CPG_obj.construct_obj.color.g;
                                 color.b=CPG_obj.construct_obj.color.b;       

                                 geometry.rmin=0;
                                 geometry.rmax=d/2; %
 
                                 geometry.start_angle_deg=0;
                                 geometry.stop_angle_deg=360;

                                 geometry.h_high=rod_end_high;
                                 geometry.h_low=rod_start_high;

                                 geometry.x_centre=X(i); % center of a cylinder
                                 geometry.y_centre=Y(i);

                                 geometry.accuracy=CPG_obj.construct_obj.accuracy;
                                  
                                 cpe=Cylinder_Phantom_Element(geometry,permittivity,element_name,element_id,color);

                                 CPG_obj.add_element(cpe);

                             end
                         else 
                             error('This is not rods_indx mode')
                         end
                     
                    otherwise,
                         error('To less or to much input elements');  
                   end % switch
              end
          end
        
        end % constructor
                
        % add phantom element to vector of elements
        [] = add_element( varargin ) 
        
        % remove element from vector (set id of element)
        remove_element(varargin)   
        
        % method use to mesh phantom
        mesh(varargin)                   
        
        % display sketch of the object
        sketch(varargin)             
        
        % discret view of phantom     
        mesh_view(varargin)       
        
        % graphical_representation
        graphical_representation(varargin);
        
        % text_properties
        text_properties(varargin)
    end
    
    methods (Access='private')
        % Hexgrid generates list of nodes of hexagonal grid
        [X Y W H A R] = hexgrid(CPG_obj,x0,y0,sizeX,sizeY,d,D,ang1,ang2)
        
        % hexplot plots circles in hexagonal grid
        hexplot(CPG_obj,x0,y0,sizeX,sizeY,d,D,ang1,mode,indx)
        
        % phantom_rod_list generates list of structures for ellipse objects
        [X,Y] = phantom_rod_list(CPG_obj,x_o,y_o,imsize,d,D, beta, eps_value)
        
        % use to draw circle
        circle(not_use,xc,yc,d,line,points)
    end
end

